#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Dec 25 16:01:11 2017

@author: albintveitankappi
"""

__all__ = ["items", "enemies", "player", "tiles", "actions", "world", "__main__"]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Dec 25 22:28:49 2017

@author: albintveitankappi
"""
import escapetheforest.items
import escapetheforest.tiles

# Things to fix:
# LeaveForrestTile cant be in the middle of the forrest, thats silly! But then again, the person playing the game 
# would never need to explore the interior... If we didnt make it absolutely neccessary. 
# 
# We could have many findStickTiles but if we already have a stick or a sharpened stick in our inventory then we shouldnt 
# Be able to pick up a new one. They should be transformed into EmptyForrestTiles
# Im thinking its time to introduce an NPC that its possible to hold a conversation with! An old man in a hut? 
# A troll, at first appearing to be a stone?

import escapetheforest.world
from escapetheforest.player import Player

def play():

    world.load_tiles()
    pc = Player()
    #The below line prints out all the attributes of the player
    #print(pc.__dict__)
    print("\n","====================THE FOREST ESCAPE========================")
    print("In this game you have two options. Escape the forest by finding the exit, or buy yourself out by selling mushrooms to the forest elf!")
    room = world.tile_exists(pc.location_x, pc.location_y)
    print(room.tile_text())
    global token
    token = 0

    #Game loop starts here:
    while pc.is_alive() and not pc.victory:
        for i in pc.inventory:
            #print(i)
            if isinstance(i, items.gold):
                if i.value >= 50:
                    print("You have gathered 50 coins and win the game!")
                    pc.victory = True

        room = world.tile_exists(pc.location_x, pc.location_y)

        if not token == 0:
            print(room.tile_text())
        token = 1
        room.modify_player(pc)
        #print("The _world dict tile instance on location {} is {}".format((pc.location_x, pc.location_y), room))

        if pc.is_alive() and not pc.victory:
            print("choose an action:\n")
            available_actions = room.available_actions(pc)
            for action in available_actions:
                print(action)
            action_input=input("Action: ")
            print("\n")
            for action in available_actions:
                if action_input == action.hotkey:
                    pc.do_action(action, **action.kwargs)
                    break
        
    if pc.is_alive() == False:
        print('You died a horrible death!')



           

if __name__ == "__main__":
    import escapetheforest.items
    import escapetheforest.world
    from escapetheforest.player import Player
    play()




import items, enemies, actions, world


class mapTile(object):
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def tile_text(self):
        raise NotImplementedError()

    def modify_player(self, player):
        raise NotImplementedError()

    def adjacent_moves(self):
        # Returns all move actions from adjacent tiles
        moves = []

        if world.tile_exists(self.x + 1, self.y):
            moves.append(actions.moveEast())

        if world.tile_exists(self.x - 1, self.y):
            moves.append(actions.moveWest())

        if world.tile_exists(self.x, self.y - 1):
            moves.append(actions.moveNorth())

        if world.tile_exists(self.x, self.y + 1):
            moves.append(actions.moveSouth())
        return moves

    def available_actions(self, pl):
        # Returns all possible actions from current tile
        moves = self.adjacent_moves()
        moves.append(actions.viewInventory())
     #   moves.append(actions.viewHP())
        moves.append(actions.viewStats())

        for i in range(0, len(pl.inventory)):
            if pl.inventory[i].name == "Wooden stick":
                moves.append(actions.sharpenStick())
                break

        for j in range(0, len(pl.inventory)):
            if pl.inventory[j].name == "Mushroom":
                moves.append(actions.eatMushroom())
                break

        moves.append(actions.killSelf())

        return moves


class startingRoom(mapTile):

    def run_once(f):
        def wrapper(*args, **kwargs):
            if not wrapper.has_run:
                wrapper.has_run = True
                return f(*args, **kwargs)
            else:
                return """You are in an unremarkable part of the forest. You should keep going and find your way out of this murky forest."""
        wrapper.has_run = False
        return wrapper
    @run_once

    def tile_text(self):
        return "\n" "\n" """Suddenly you wake up and find yourself in a dark forest... You look around, wondering how you got there. 
It's damp and cold, so you decide to get moving. """

    def modify_player(self, player):
        # "Room has no action on player"
        pass

class emptyForrestTile(mapTile):
    def tile_text(self):
        return """Another unremarkable part of the forrest. You must forge onwards.
        """
    def modify_player(self, player):
        # Room has no action on player
        pass

class lootRoom(mapTile):
    def __init__(self, x, y, item):
        self.taken = False
        self.item = item
        super(lootRoom, self).__init__(x, y)

    def add_loot(self, player):
        player.inventory.append(self.item)

    def modify_player(self, player):
        if (self.taken == False):
            self.add_loot(player)
            self.taken = True

class NPCtile(mapTile):
    def __init__(self, x, y, NPC):
        self.NPC=NPC
        self.metNPC = False
        super(NPCtile, self).__init__(x, y)



class traderTile(NPCtile):
    def __init__(self, x, y):
        super(traderTile, self).__init__(x, y, enemies.trader())

    def tile_text(self):
        if (self.NPC.is_alive()) and (self.metNPC == False):
            self.metNPC = True
            print("A small figure with a big ears suddenly stands in front of you on a stump. \n'Howdy! I'm a forest elf and I wanna buy mushrooms, I love them! You are welcome to sell them to me if you have any!'")
        elif (self.NPC.is_alive()) and (self.metNPC == True):
            return "The mushroom-loving elf stands there on the stump looking at you, " \
                   "waiting for you to sell him some mushrooms."
        else:
            return """The corpse of a dead elf rots on the ground.
            """

    def modify_player(self, the_player):
        # "Room has no action on player"
        pass

    def available_actions(self, pl):

        moves = self.adjacent_moves()
        moves.append(actions.viewInventory())
        moves.append(actions.viewStats())

        for i in range(0, len(pl.inventory)):
            if pl.inventory[i].name == "Wooden stick":
                moves.append(actions.sharpenStick())
                break

        for j in range(0, len(pl.inventory)):
            if pl.inventory[j].name == "Mushroom":
                moves.append(actions.eatMushroom())
                break

        for j in range(0, len(pl.inventory)):
            if (actions.sellMushroom() not in moves):
                #print("The action was not found")
                if pl.inventory[j].name == "Mushroom":
                   # print("Mushroom exists, adding the action")
                    moves.append(actions.sellMushroom())
                    break
        moves.append(actions.killSelf())
        return moves



class enemyRoom(mapTile):
    def __init__(self, x, y, enemy):
        self.enemy = enemy
        self.hasAttacked = False
        self.metEnemy = False
        super(enemyRoom, self).__init__(x, y)

    def modify_player(self, the_player):
        if (self.enemy.is_alive()) and (self.enemy.isaggressive):
            the_player.hp = the_player.hp - self.enemy.damage
            if the_player.hp <= 0:
                print("The enemy does {} damage. You have {} hp remaining.".format(self.enemy.damage, the_player.hp))
                print("You fall to the ground, trying to crawl away from the enemy, but the enemy strikes one last blow leaving you dead on the ground...")

            else:
                print("The enemy does {} damage. You have {} hp remaining.".format(self.enemy.damage, the_player.hp))


    def available_actions(self, pl):
        if self.enemy.is_alive():
            return [actions.flee(tile=self), actions.attack(enemy=self.enemy)]
        else:
            moves = self.adjacent_moves()
            moves.append(actions.viewInventory())
            moves.append(actions.viewStats())

            for i in range(0, len(pl.inventory)):
                if pl.inventory[i].name == "Wooden stick":
                    moves.append(actions.sharpenStick())
                    break

            for j in range(0, len(pl.inventory)):
                if pl.inventory[j].name == "Mushroom":
                    moves.append(actions.eatMushroom())
                    break

            moves.append(actions.killSelf())
            return moves

class bigSpiderTile(enemyRoom):
    def __init__(self, x,y):
        #self.hasAttacked = False
        super(bigSpiderTile, self).__init__(x,y, enemies.bigSpider())

    def tile_text(self):
        if self.enemy.is_alive() and self.hasAttacked == False:
            self.hasAttacked = True
            return """A big spider comes to bite you!""""\n"

        elif self.hasAttacked == True:
            return """The big spider attacks again!"""
        else:
            return """A big dead spider lies on the ground here"""

class oldManTile(enemyRoom):
    def __init__(self, x, y):
        #self.metOldman= False
        super(oldManTile, self).__init__(x, y, enemies.crazyOldMan())

    def tile_text(self):
        if (self.enemy.is_alive()) and (self.metEnemy == False):
            self.metEnemy = True
            return """A crazy old man with red eyes and a big bears suddenly stands before you. He has an axe in his hand and runs towards you to hit you!
            """
        elif (self.enemy.is_alive()) and (self.metEnemy == True):
            return "The old man hits you with his axe."
        else:
            return """The corpse of an old dead man rots on the ground.
            """

class banditTile(enemyRoom):
    def __init__(self, x, y):
        super(banditTile, self).__init__(x, y, enemies.bandit())


    def tile_text(self):
        if (self.enemy.is_alive() and self.enemy.isaggressive == True):
            self.metEnemy = True
            return """The gives out a roar and strikes you, 'RRAAAAAHH!!'
            """

        elif (self.enemy.is_alive() and self.enemy.isaggressive == False and self.metEnemy == False):
            self.metEnemy = True
            return ("""A bandit approaches you! Hey hey, who do we have here... Don't you know that there is a fee of 10G to pass through here? Pay up, or die!
            """)

        elif (self.metEnemy == True) and (self.enemy.is_alive()) and (self.enemy.isaggressive == False):
            return """The bandit just stands and looks at you with an evil grin on his face"""

        else:
            return """The corpse of a dead man rots on the ground.
                        """

    def available_actions(self, pl):
        if (self.enemy.isaggressive == False) and self.enemy.is_alive():
            return [actions.giveTenG(enemy=self.enemy), actions.flee(tile=self), actions.attack(enemy=self.enemy)]

        elif (self.enemy.isaggressive==True) and self.enemy.is_alive():
            return [actions.flee(tile=self), actions.attack(enemy=self.enemy)]

        elif (self.enemy.isaggressive==True) and (self.enemy.is_alive()) and (self.metBandit == True):
            return mapTile.available_actions(self, pl)

        elif not self.enemy.is_alive():
            #This mf returns only the available actions from the maptile class, fuk yaaaaah!!
            return mapTile.available_actions(self, pl)


class ringleaderTile(enemyRoom):
    def __init__(self, x, y):
        super(ringleaderTile, self).__init__(x, y, enemies.ringLeader())

    def tile_text(self):
        if (self.enemy.is_alive()) and (self.enemy.isaggressive == True) and (self.hasAttacked == False):
            self.hasAttacked = True
            return """'RRAAAAAHH!!'
            """
        elif self.hasAttacked == True:
            return """The bandit ringleader attacks again!"""

        elif (self.enemy.is_alive()) and (self.enemy.isaggressive == False):
            return ("""The bandit ringleader approaches you!
'So, you have come to visit our dark forest? We dont want people snooping around here. Pay up 30 gold or die!'
            """)

        else:
            pass


    def available_actions(self, pl):
        if (self.enemy.isaggressive == False) and self.enemy.is_alive():
            return [actions.giveAllMoney(enemy=self.enemy), actions.flee(tile=self), actions.attack(enemy=self.enemy)]
        elif (self.enemy.isaggressive==True) and self.enemy.is_alive():
            return [actions.flee(tile=self), actions.attack(enemy=self.enemy)]

        elif not self.enemy.is_alive():
            #This returns only the available actions from the maptile class:
            return mapTile.available_actions(self, pl)



class findKnifeTile(lootRoom):
    def __init__(self, x, y):
        super(findKnifeTile, self).__init__(x, y, items.knife())

    def tile_text(self):
        if self.taken == False:
            return """Your notice something on the ground. It's a knife! You pick it up.
        """
        else:
            return """
        Another unremarkable part of the forrest. You must forge onwards.
        """


###Create more rooms for finding gold and stumbling upon the ringleader!

class leaveForrestTile(mapTile):

    def tile_text(self):
        return """You see a field crossed by a road glimpsing between the trees.
    You are outside of the forest!"""

    def modify_player(self, player):
        player.victory = True


class findStickTile(lootRoom):

    def __init__(self, x, y):
        super(findStickTile, self).__init__(x, y, items.stick())

    def tile_text(self):
        if self.taken == False:
            return """You see a piece of wood, it might make a decent weapon and you pick it up. """
        else:
            return """Another unremarkable part of the forrest. You must forge onwards."""


class findMushRoomTile(lootRoom):
    token = 0
    def __init__(self, x, y):
        super(findMushRoomTile, self).__init__(x, y, items.mushRoom())

    def tile_text(self):
        if self.taken == False:
            return """Your notice something on the ground. It's a strange looking mushroom! You pick it up."""

        elif self.token == 0:
            self.token = 1
            return """Another unremarkable part of the forrest. You must forge onwards."""

        else:
            return """Another unremarkable part of the forrest. You must forge onwards."""









"""item classes are found in this script"""

class item(object):
    def __init__(self, name, description, value):
        self.name = name
        self.description = description
        self.value = value

    def __str__(self):
        return "{0}\n Value is:{1}\n {2}\n".format(self.name, self.value, self.description)



class book(item):
    def __init__(self, title):
        self.title = title
        super(book, self).__init__(name = "Book",
                                   description="An old dusty book",
                                   title = "How to get strong with 5 easy tricks",
                                   value = 20)



class gold(item):
    def __init__(self, amt):
        self.amt = amt
        super(gold, self).__init__(name="Gold",
                                   description="Gold coins.".format(str(self.amt)),
                                   value=self.amt)


class weapon(item):
    def __init__(self, name, description, value, damage, rangy):
        self.damage = damage
        self.rangy = rangy
        super(weapon, self).__init__(name, description, value)

    def __str__(self):
        return "{0}\n {1}\n Value is: {2}\n Damage: {3}\n".format(self.name, self.description, self.value, self.damage)


class fists(weapon):
    def __init__(self):
        super(fists, self).__init__(name="Fists",
                                    description="Your medium sized fists",
                                    value=0,
                                    damage=1,
                                    rangy=2)

    def __str__(self):
        return """You have no weapon in your bag, you will have to rely on your fists in case of any danger."""

class knife(weapon):
    def __init__(self):
        super(knife, self).__init__(name="knife",
                                    description="A shorter type knife suitable for wood carving",
                                    value=5,
                                    damage=4,
                                    rangy=1)


class sharpenedStick(weapon):
    def __init__(self):
        super(sharpenedStick, self).__init__(name="Sharpened stick",
                                             description="A long stick of hard wood, sharpened to be pointy in one end.",
                                             value=0,
                                             damage=6,
                                             rangy=3)


class stick(weapon):
    def __init__(self):
        super(stick, self).__init__(name="Wooden stick",
                                    description="Just a piece of wood", value=0,
                                    damage=3,
                                    rangy=1)


## Make a new class of edibles?? 

class edible(item):
    def __init__(self, name, description, value, hpadd, dmgadd):
        self.hpadd = hpadd
        self.dmgadd = dmgadd
        super(edible, self).__init__(name, description, value)

    def __str__(self):
        return "{0}\n {1}\n Value is: {2}\n Hp add: {3}\n Damage add: {4}\n".format(self.name,
                                                                                    self.description,
                                                                                    self.value,
                                                                                    self.hpadd,
                                                                                    self.dmgadd)


class mushRoom(edible):
    def __init__(self):
        super(mushRoom, self).__init__(
            name="Mushroom",
            description="An odd looking mushroom...",
            value=10,
            hpadd=2,
            dmgadd=2

        )




"""
This is the player file, having a single class called player, where all the methods (activities) of the character is coded!!
"""

import items, world

import random


class Player(object):
    def __init__(self):
        self.inventory = [items.gold(5), items.fists()]
        self.hp = 20
        self.strength= 2
        self.location_x, \
        self.location_y = world.starting_position
        self.victory = False



    def has_stick(self):
            if items.sharpenedStick in self.inventory:
                return True
                print("got that stick already")
            else:
                return False

    def is_alive(self):
        return self.hp > 0

    def print_inventory(self):
        if len(self.inventory) == 1 and (self.inventory[0].name == "Fists"):
            print("You open your backpack and find nothing inside","\n")
        else:
            print("You open your backpack and find the following items:","\n")
            for item in self.inventory:
                print(item, "\n")

    #def print_hp(self):
    #    print("Your HP is:" + str(self.hp), "\n")


    def print_stats(self):
        print("Your HP is: " + str(self.hp), "\n")
        print("Your Strength is: " + str(self.strength), "\n")

    def flee(self, tile):
        available_moves = tile.adjacent_moves()
        r = random.randint(0, len(available_moves) - 1)
        self.do_action(available_moves[r])

    def move(self, dx, dy):
        self.location_x += dx
        self.location_y += dy
        #The following prints the intro text at the current tile where the player is located:
        #print(world.tile_exists(self.location_x, self.location_y).intro_text())

    def move_north(self):
        self.move(dx=0, dy=-1)

    def move_south(self):
        self.move(dx=0, dy=1)

    def move_east(self):
        self.move(dx=1, dy=0)

    def move_west(self):
        self.move(dx=-1, dy=0)

    def sellAmushroom(self):
        global goldExists
        goldExists = False
       # while token == 0:
        #Check for mushroom and remove it(its sold after all)
        for j in self.inventory:
            if isinstance(j, items.mushRoom):
                self.inventory.remove(j)

        #Check for gold, if gold exists, return: add value of the mushroom to the gold.
        for i in self.inventory:
            if isinstance(i, items.gold):
                goldExists=True
                print("Mushroom sold for", j.value)
                i.value += j.value
                if i.value >= 50:
                    print("Oooh, I sense you have much gold in your pockets human! It would be a shame if the bandits of this forests took it... Let me teleport you out of this horrible place and into your warm and safe bed instead!")

        if goldExists == False:
            self.inventory.append(items.gold(j.value))
            print("Mushroom sold for {}g".format(j.value))

    def giveAllMoney(self, enemy):

        for i in self.inventory:
            if isinstance(i, items.gold) and (i.amt >= 30):
                global givenAmt
                givenAmt = i.amt
                self.inventory.remove(i)
                print("You give out all your money")
                if enemy.isaggressive == True:
                    enemy.calm()
                print("He calmed down")
                break
            elif isinstance(i, items.gold)and (i.amt < 30):
                print("The money isn't enough...")
                enemy.provoke()
                break
            else:
                print("When I reach into my pocket I cant find any money!")
                enemy.provoke()
                break


    def giveTenG(self, enemy):

        for i in self.inventory:
            if isinstance(i, items.gold) and (i.amt >= 10):
                i.amt-= 10
                print("You pay 10G")
                if enemy.isaggressive == True:
                    enemy.calm()
                print("Your enemy has calmed down")
                break
            elif isinstance(i, items.gold)and (i.amt < 10):
                print("The money isn't enough...")
                enemy.provoke()
                break
            else:
                print("When I reach into my pocket I cant find any money!")
                enemy.provoke()
                break

    def returnMoney(self):
        self.inventory.append(items.gold(givenAmt))
        print("You took back your money")


    def attack(self, enemy):
        if not enemy.isaggressive:
            print("You surprise the enemy by attacking it.")
            enemy.provoke()
        best_weapon = None
        max_dmg = 0

        for i in self.inventory:
            if isinstance(i, items.weapon):
                if i.damage > max_dmg:
                    max_dmg = i.damage
                    best_weapon = i

        curdmg = random.randint(best_weapon.damage - best_weapon.rangy, best_weapon.damage)
        curdmg += self.strength
        enemy.hp -= curdmg
        print("You use {0} against {1}, causing {2} damage".format(best_weapon.name, enemy.name, curdmg))

        if not enemy.is_alive():
            print("Wham! You smite the {0} to the ground and remain victorious!".format(enemy.name))

        else:
            print("{0} hp is {1}".format(enemy.name, enemy.hp))

    def do_action(self, action, **kwargs):
        action_method = getattr(self, action.method.__name__)
        if action_method:
            action_method(**kwargs)

    def sharpen(self):
        hassharp = False
        val = len(self.inventory)
        i = 0
        while i < val:
            if self.inventory[i].name == "Wooden stick":
                del self.inventory[i]
                val = -1
                i = -1
            # elif self.inventory[i].name == "Sharpened stick":
            #     hassharp=True
            i = i + 1
        if hassharp == False:
            self.inventory.append(items.sharpenedStick())
        print("You sharpen the wooden stick into a dangerous looking weapon!""\n")

    def killSelfie(self):
        self.hp = 0

    def eatMushroom(self):
        val = len(self.inventory)
        i = 0
        healthEffect = 0
        strengthEffect = 0

        while i < val:
            if self.inventory[i].name == "Mushroom":
                healthEffect = self.inventory[i].hpadd
                strengthEffect = self.inventory[i].dmgadd
                del self.inventory[i]
                break

            i = i + 1

        self.hp += healthEffect
        self.strength += strengthEffect

        print("You eat the strange mushroom in one big bite. You feel healthy, and your arms seem bigger!""\n")
       # game.token = 1






from player import Player



class action(object):
    def __init__(self, method, name, hotkey, **kwargs):
        self.method = method
        self.name = name
        self.hotkey = hotkey
        self.kwargs = kwargs
        
    def __str__(self):
        return "{}: {}".format(self.hotkey, self.name)
        
        
class moveNorth(action):
    def __init__(self):
        super(moveNorth,self).__init__(method=Player.move_north, name="Move north", hotkey = "n")
        
class moveSouth(action):
    def __init__(self):
        super(moveSouth,self).__init__(method=Player.move_south, name="Move south", hotkey = "s")
        
class moveEast(action):
    def __init__(self):
        super(moveEast,self).__init__(method=Player.move_east, name="Move east", hotkey = "e")
        
class moveWest(action):
    def __init__(self):
        super(moveWest,self).__init__(method=Player.move_west, name="Move west", hotkey = "w")
        
        
class viewInventory(action):
    #Prints the players inventory
    def __init__(self):
        super(viewInventory,self).__init__(method= Player.print_inventory, name = "view inventory", hotkey = "i")


#class viewHP(action):
#    #Prints the players inventory
#    def __init__(self):
#        super(viewHP,self).__init__(method= Player.print_hp, name = "view HP", hotkey = "h")

class viewStats(action):
    #Prints the players inventory
    def __init__(self):
        super(viewStats,self).__init__(method= Player.print_stats, name = "view stats", hotkey = "l")

class attack(action):
    def __init__(self, enemy):
        super(attack,self).__init__(method=Player.attack, name = "Attack", hotkey = "a", enemy = enemy)

        
class flee(action):
    def __init__(self, tile):
        super(flee,self).__init__(method=Player.flee, name = "Flee!", hotkey = "f", tile = tile)
        

        """New functionality"""
        
class sharpenStick(action):
    #Makes the player sharpen a stick and add the sharpenedStick weapon to the inventory
    def __init__(self):
        super(sharpenStick,self).__init__(method=Player.sharpen, name = "Sharpen the wooden stick", hotkey = "g")

        
class killSelf(action):
    def __init__(self):
        super(killSelf,self).__init__(method=Player.killSelfie, name = "This is no fun 😞", hotkey = "m")


## Add eat edible functionality:

class eatMushroom(action):
    def __init__(self):
        super(eatMushroom,self).__init__(method=Player.eatMushroom, name = "Eat mushroom", hotkey = "y")

# Add give money action

class giveAllMoney(action):
    def __init__(self, enemy):
        super(giveAllMoney, self).__init__(method=Player.giveAllMoney, name="Pay the money", hotkey = "p", enemy = enemy)


class giveTenG(action):
    def __init__(self, enemy):
        super(giveTenG, self).__init__(method=Player.giveTenG, name="Pay the money", hotkey = "p", enemy = enemy)



class sellMushroom(action):
    def __init__(self):
        super(sellMushroom, self).__init__(method=Player.sellAmushroom, name="Sell mushroom", hotkey = "g")




"""This is where all the enemies reside """

class NPC(object):
    def __init__(self, name, hp):
        self.name=name
        self.hp = hp

    def is_alive(self):
        return self.hp > 0

# THe following are NPCs:

class trader(NPC):
    def __init__(self):
        super(trader, self).__init__(name="Tradesman", hp=10)



#Following are enemy class and enemies:
class enemy(object):
    def __init__(self, name, hp, damage, isaggressive):
        self.isaggressive = isaggressive
        self.name = name
        self.hp = hp
        self.damage = damage

        
    def is_alive(self):
        return self.hp > 0

    def provoke(self):
        self.isaggressive = True
        print("You have provoked the enemy to aggression!")

    def calm(self):
        self.isaggressive = False
        print("Enemy has calmed down and is no longer prone to attacking")

class bandit(enemy):
    def __init__(self):
        super(bandit,self).__init__(name = "Bandit", hp = 10, damage = 3, isaggressive=False)

class ringLeader(enemy):
    def __init__(self):
        super(ringLeader, self).__init__(name = "Bandit ringleader", hp = 15, damage = 4, isaggressive=False)

class bigSpider(enemy):
    def __init__(self):
        super(bigSpider, self).__init__(name="Big spider", hp=6, damage=2, isaggressive=True)

class crazyOldMan(enemy):
    def __init__(self):
        super(crazyOldMan,self).__init__(name = "Crazy old man", hp = 10, damage = 5, isaggressive=True)
        





import tiles

"""This is the actual world """
import random

_world = {}
starting_position = (0, 0)

#Now, this function goes through the map tile and creates a dictionary that contains the coordinates and the tile name


def load_tiles():
    """Parses a file that describes the world space into the _world object"""
    with open('resources/map.txt', 'r') as f:
        rows = f.readlines()
        row=[]

        # the following lines takes each row of the map.txt file and appends them to a long list. Then it shuffles the list randomly and then it
        # creates a new list with all the tiles randomly shuffled.

  #      for i in range(len(rows)):
  #          row.extend(rows[i].replace('\n','').split('\t'))
  #          #print(rows[i])
  #          #print(row)
  #          random.shuffle(row)
  #          val=0
  #          row2=[]
  #          while val<len(row):
  #              row2.append('\t'.join(row[val:val+4])+'\n')
  #              #print(row2)
  #              val+=4

  #  rows=row2
    x_max = len(rows[0].split('\t')) # Assumes all rows contain the same number of tabs

    for y in range(len(rows)):
        cols = rows[y].split('\t')

        for x in range(x_max):
            tile_name = cols[x].replace('\n', '') # Windows users may need to replace '\r\n'

            #This just says that if the tile_name is startingroom, that should be our starting position.
            if tile_name == 'startingRoom':
                global starting_position
                starting_position = (x, y)

            #The variable _world is a dictionary that maps a coordinate pair to a tile. So the code _world[(x, y)]=....
            #creates the key (i.e. the coordinate pair) of the dictionary. If the cell is
            #an empty string, we don’t want to store a tile in it’s place which is why we have the code None if tile_name == ''.
            #However, if the cell does contain a name, we want to actually create a tile of that type. The getattr method is
            #built into Python and lets us reflect into the tile module and find the class whose name matches tile_name.
            #Finally the (x, y) passes the coordinates to the constructor of the tile.

            _world[(x, y)] = None if tile_name == '' else getattr(__import__('tiles'), tile_name)(x, y)

            if tile_name == '':
                _world[(x,y)] = None

            else:
                _world[(x, y)] = getattr(__import__('tiles'), tile_name)(x, y)



"""
    rlist=[0, len(rows)-1]
    print(rlist)

    print(rows)
    for key, value in _world.items():
        if(isinstance(value, tiles.leaveForrestTile)):
            print(key,value)
            print(key[0])
            if (key[1] != 0) and (key[1] != len(rows)-1):
                print("boo")
                nkey = (random.randint(0,3),random.choice(rlist))
                print(nkey)
                _world[key]= _world[nkey]
                print(_world[key])
                print(_world[nkey])
                print(_world)
                """








def emptify_tile(x,y):
    _world[(x, y)] = tiles.emptyForrestTile(x, y)
    return _world[(x, y)]


def tile_exists(x,y):
    """Returns the tile at the given coordinates or None if there is no tile.
    :param x: the x-coordinate in the worldspace
    :param y: the y-coordinate in the worldspace
    :return: the tile at the given coordinates or None if there is no tile
    """
    return _world.get((x,y))
